using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using MVR.FileManagementSecure;

namespace lynyrd2473 {

	/// <summary>
	/// Tracker4Camera01
	/// V1.0
	/// </summary>
	public class Tracker4Camera01 : MVRScript {

		protected const string strSavePath = "Saves\\PluginData\\lynyrd2473\\Tracker4Camera";
		protected JSONStorableBool bEnable;

		protected Atom receivingAtom;

		protected JSONStorableStringChooser atomJSON;
		protected JSONStorableStringChooser receiverJSON;

		protected JSONStorableStringChooser modeJSON;

		protected JSONStorableBool	bPositionX;
		protected JSONStorableBool	bPositionY;
		protected JSONStorableBool	bPositionZ;

		protected JSONStorableFloat fDistanceX;
		protected JSONStorableFloat	fDistanceY;
		protected JSONStorableFloat	fDistanceZ;

		protected JSONStorableBool	bRotationY;
		protected JSONStorableBool	bRotationX;
		// 2023/09/08 Flip ǉ
		protected JSONStorableBool	bRotationFlip;

		protected UIDynamicToggle dtx;
		protected UIDynamicSlider dsx;
		protected UIDynamicToggle dty;
		protected UIDynamicSlider dsy;
		protected UIDynamicToggle dtz;
		protected UIDynamicSlider dsz;

		protected UIDynamicToggle dtry;
		protected UIDynamicToggle dtrx;
		protected UIDynamicToggle dtrFlip;

		protected UIDynamic sp1;
		protected UIDynamic sp2;

		public override void Init() {
			try {
				// put init code in here
				//SuperController.LogMessage("Tracker4Camera01 Loaded");

				bEnable = new JSONStorableBool("Enable", false);
				bEnable.storeType = JSONStorableParam.StoreType.Full;
				RegisterBool(bEnable);
				CreateToggle(bEnable, true);

				UIDynamicButton dbSave = CreateButton("Save Preset");
				UIDynamicButton dbLoad = CreateButton("Load Preset");

				dbSave.button.onClick.AddListener(() => {
					SuperController.singleton.GetMediaPathDialog(SaveFileSelected, "tr4c", GetPluginDataPath(), false, true, false);
					uFileBrowser.FileBrowser browser = SuperController.singleton.mediaFileBrowserUI;
					browser.SetTextEntry(true);
					browser.fileEntryField.text = String.Format("{0}.{1}", ((int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds).ToString(), "tr4c");
					browser.ActivateFileNameField();

				});

				dbLoad.button.onClick.AddListener(() => {
					SuperController.singleton.GetMediaPathDialog(LoadFileSelected, "tr4c", GetPluginDataPath(), false, true, false);
				});

				// make atom selector
				atomJSON = new JSONStorableStringChooser("atom", SuperController.singleton.GetAtomUIDs(), null, "Atom", SyncAtom);
				atomJSON.representsAtomUid = true;
				atomJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterStringChooser(atomJSON);
				SyncAtomChocies();
				UIDynamicPopup dp = CreateFilterablePopup(atomJSON);
				dp.popupPanelHeight = 1100f;

				// make receiver selector
				receiverJSON = new JSONStorableStringChooser("receiver", null, null, "Receiver", SyncReceiver);
				receiverJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterStringChooser(receiverJSON);
				dp = CreateFilterablePopup(receiverJSON);
				dp.popupPanelHeight = 960f;

				List<string> ModeChoices = new List<string>();
				ModeChoices.Add("Position");
				ModeChoices.Add("Rotation");

				modeJSON	= new JSONStorableStringChooser("mode", ModeChoices, "Position", "Mode", SyncMode);
				modeJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterStringChooser(modeJSON);
				dp = CreateFilterablePopup(modeJSON, true);

				bPositionX = new JSONStorableBool("PositionX", true);
				bPositionX.storeType = JSONStorableParam.StoreType.Full;
				RegisterBool(bPositionX);
				bPositionY = new JSONStorableBool("PositionY", false);
				bPositionY.storeType = JSONStorableParam.StoreType.Full;
				RegisterBool(bPositionY);
				bPositionZ = new JSONStorableBool("PositionZ", true);
				bPositionZ.storeType = JSONStorableParam.StoreType.Full;
				RegisterBool(bPositionZ);

				fDistanceX = new JSONStorableFloat("DistanceX", 0f, -100f, 100f, true);
				fDistanceX.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(fDistanceX);
				fDistanceY = new JSONStorableFloat("DistanceY", 0f, -100f, 100f, true);
				fDistanceY.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(fDistanceY);
				fDistanceZ = new JSONStorableFloat("DistanceZ", 0f, -100f, 100f, true);
				fDistanceZ.storeType=JSONStorableParam.StoreType.Full;
				RegisterFloat(fDistanceZ);

				bRotationY = new JSONStorableBool("RotationY", true);
				bRotationY.storeType=JSONStorableParam.StoreType.Full;
				RegisterBool(bRotationY);
				bRotationX = new JSONStorableBool("RotationX", false);
				bRotationX.storeType=JSONStorableParam.StoreType.Full;
				RegisterBool(bRotationX);

				// 2023/09/08 Flip ǉ
				bRotationFlip = new JSONStorableBool("RotationFlip", false);
				bRotationFlip.storeType=JSONStorableParam.StoreType.Full;
				RegisterBool(bRotationFlip);

				CreateSpacer(true);

				SyncMode(modeJSON.val);
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		void Start() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		void Update() {
			try {

/*
                if ((Input.GetKey(KeyCode.LeftControl) || Input.GetKey(KeyCode.RightControl)))
                {
                    if (Input.GetKeyDown(KeyCode.X) )
                    {                        

                    }
                }
*/

				if (!bEnable.val)
                {
					return;
                }
				if (atomJSON.val == null)
                {
					return;
				}
				if (receiverJSON.val == null)
                {
					return;
                }

	            FreeControllerV3 CameraController = containingAtom.GetStorableByID("control") as FreeControllerV3;
				Vector3 CameraPos = CameraController.transform.position;

				Atom atom = SuperController.singleton.GetAtomByUid(atomJSON.val);
                if (atom != null)
                {
                    JSONStorable targetreceiver = atom.GetStorableByID(receiverJSON.val);
                    Vector3 TargetPos = targetreceiver.transform.position;

                    if (targetreceiver != null)
                    {
                        if (modeJSON.val == "Position")
                        {

                            if (bPositionX.val)
                            {
                                CameraPos[0] = TargetPos.x + fDistanceX.val;
                            }
                            if (bPositionY.val)
                            {
                                CameraPos[1] = TargetPos.y + fDistanceY.val;
                            }
                            if (bPositionZ.val)
                            {
                                CameraPos[2] = TargetPos.z + fDistanceZ.val;
                            }
                            CameraController.transform.transform.position = CameraPos;
                        }
                        else
                        {
							// Rotation
							Vector3 CameraAngle = CameraController.transform.localEulerAngles;
                            float angleX = CameraAngle.x;
                            float angleY = CameraAngle.y;
                            float angleZ = CameraAngle.z;

                            if (bRotationY.val)
                            {
                                angleY = GetAngleY(CameraPos, TargetPos);
                            }
                            if (bRotationX.val)
                            {
                                angleX = GetAngleX(CameraPos, TargetPos);
                            }
							
                            CameraController.transform.rotation = Quaternion.Euler(angleX, angleY, angleZ);
                        }
                    }
                }
            }
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}


		/// <summary>
		/// Callback Function
		/// </summary>
		protected void SyncAtomChocies()
		{
			List<string> atomChoices = new List<string>();
			foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
			{
				if (containingAtom.name != atomUID) {
					atomChoices.Add(atomUID);
				}
			}
			atomJSON.choices = atomChoices;
		}

		protected void SyncAtom(string atomUID)
		{
			List<string> receiverChoices = new List<string>();
			if (atomUID != null)
			{
				receivingAtom = SuperController.singleton.GetAtomByUid(atomUID);
				if (receivingAtom != null)
				{
					foreach (string receiverChoice in receivingAtom.GetStorableIDs())
					{
						JSONStorable receiver = receivingAtom.GetStorableByID(receiverChoice);
						if (receiver != null)
                        {
							if (receivingAtom.type == "Person")
							{
								if (receiver.GetType().ToString() == "DAZBone")
								{
									receiverChoices.Add(receiverChoice);
								}
							}
							else
							{
								receiverChoices.Add(receiverChoice);
							}
						}
					}
				}
			}
			else
			{
				receivingAtom = null;
			}
			receiverJSON.choices = receiverChoices;
		}

		protected void SyncReceiver(string receiverID)
		{
		}

		//[hICallback
		protected void SyncMode(string mode) 
		{
			if (mode == "Position")
            {
				if (dtry != null)
                {
					RemoveToggle(dtry);
                }
				if (dtrx != null)
				{
					RemoveToggle(dtrx);
				}

				// 2023/09/08 Flip ǉ
				if (dtrFlip != null)
				{
					RemoveToggle(dtrFlip);
				}

				dtx = CreateToggle(bPositionX, true);
				dtx.label = "Position:X";
				dsx = CreateSlider(fDistanceX, true);
				dsx.label = "Distance:X";

				sp1 = CreateSpacer(true);

				dty = CreateToggle(bPositionY, true);
				dty.label = "Position:Y";
				dsy = CreateSlider(fDistanceY, true);
				dsy.label = "Distance:Y";

				sp2= CreateSpacer(true);

				dtz = CreateToggle(bPositionZ, true);
				dtz.label = "Position:Z";
				dsz = CreateSlider(fDistanceZ, true);
				dsz.label = "Distance:Z";
			}
			else
            {
				if (sp1 != null)
                {
					RemoveSpacer(sp1);
                }
				if (sp2 != null)
				{
					RemoveSpacer(sp2);
				}
				if (dtx != null)
				{
					RemoveToggle(dtx);
				}
				if (dty != null)
				{
					RemoveToggle(dty);
				}
				if (dtz != null)
				{
					RemoveToggle(dtz);
				}
				if (dsx != null)
				{
					RemoveSlider(dsx);
				}
				if (dsy != null)
				{
					RemoveSlider(dsy);
				}
				if (dsz != null)
				{
					RemoveSlider(dsz);
				}

				dtry = CreateToggle(bRotationY, true);
				dtry.label = "Rotation:Y";

				dtrx = CreateToggle(bRotationX, true);
				dtrx.label = "Rotation:X";

				// 2023/09/08 Flip ǉ
				dtrFlip = CreateToggle(bRotationFlip, true);
				dtrFlip.label = "Rotation:Flip";

			}

		}

		private void SaveFileSelected(string fileName)
		{
			try
			{
				if (fileName != "")
				{
					SuperController.singleton.SaveJSON(this.GetJSON(true, true, true), fileName);
				}
			}
			catch (Exception e) { SuperController.LogError("Exception caught: " + e); }
		}

		private void LoadFileSelected(string fileName)
		{
			try
			{
				if (fileName != "")
				{
					if (FileManagerSecure.FileExists(fileName))
					{
						JSONNode loadJson = SuperController.singleton.LoadJSON(fileName);
						this.RestoreFromJSON((JSONClass)loadJson);
					}
				}
			}
			catch (Exception e) { SuperController.LogError("Exception caught: " + e); }
		}

		// Y2_Ԃ̊px擾
		//
		//           0
		//           
		//	270  J  90		iォ猩}j
		//           
		//          180
		//
		float GetAngleY(Vector3 cameraPos, Vector3 target)
		{
			Vector3 dt = target - cameraPos;
			if (dt.x == 0)	{
				return 0;
			}
			float rad = Mathf.Atan2(dt.x, dt.z);
			float degree = rad * Mathf.Rad2Deg;

			// ʂ180(-180)180ɂ
			if (degree < 0)
			{
				degree = 360 + degree;
			}

			// ʂ0ɂ
			degree = degree - 180;
			if (degree < 0)
			{
				degree = 360 + degree;
			}

			// 2023/09/08 Flip ǉ
			if (bRotationFlip.val)
			{
				degree = flip(degree);
			}

			return degree;
		}


		// X2_Ԃ̊px擾
		//
		//           90
		//           
		//	  0  J  0		i猩}j
		//           
		//          -90
		//
		float GetAngleX(Vector3 cameraPos, Vector3 target)
		{
			Vector3 dt = target - cameraPos;
			float xz  = (float)Math.Sqrt(dt.z * dt.z + dt.x * dt.x);
			if (xz == dt.y)	{
				return 0;
			}
			float rad = Mathf.Atan2(dt.y, xz);
			float degree = rad * Mathf.Rad2Deg;

				// 2023/09/08 Flip ǉ
			if (bRotationFlip.val)
			{
				degree *= -1;
			}

			return degree;
		}

		protected string GetPluginDataPath()
		{
			bool exists = FileManagerSecure.DirectoryExists(strSavePath);
			if (!exists)
            {
				FileManagerSecure.CreateDirectory(strSavePath);
			}
			return strSavePath;
		}

		// ]	2023/09/08	ǉ
		float flip(float src)
		{
			float	dst	= src + 180;
			if (dst > 360)	{
				dst	-= 360;
			}
			return	dst;
		}


	}

}
